--copied from UIParent.lua, change field names

--if UICoreFrameFlash then return end

local frameFlashManager = CreateFrame("FRAME");

local FLASHFRAMES = {}
local UIFrameFlashTimers = {};
local UIFrameFlashTimerRefCount = {};

-- Function to start a frame flashing
function UICoreFrameFlash(frame, fadeInTime, fadeOutTime, flashDuration, showWhenDone, flashInHoldTime, flashOutHoldTime, syncId)
	if ( frame ) then
		local index = 1;
		-- If frame is already set to flash then return
		while FLASHFRAMES[index] do
			if ( FLASHFRAMES[index] == frame ) then
				return;
			end
			index = index + 1;
		end

		if (syncId) then
			frame._syncId = syncId;
			if (UIFrameFlashTimers[syncId] == nil) then
				UIFrameFlashTimers[syncId] = 0;
				UIFrameFlashTimerRefCount[syncId] = 0;
			end
			UIFrameFlashTimerRefCount[syncId] = UIFrameFlashTimerRefCount[syncId]+1;
		else
			frame._syncId = nil;
		end
		
		-- Time it takes to fade in a flashing frame
		frame._fadeInTime = fadeInTime;
		-- Time it takes to fade out a flashing frame
		frame._fadeOutTime = fadeOutTime;
		-- How long to keep the frame flashing
		frame._flashDuration = flashDuration;
		-- Show the flashing frame when the fadeOutTime has passed
		frame._showWhenDone = showWhenDone;
		-- Internal timer
		frame._flashTimer = 0;
		-- How long to hold the faded in state
		frame._flashInHoldTime = flashInHoldTime;
		-- How long to hold the faded out state
		frame._flashOutHoldTime = flashOutHoldTime;
		
		tinsert(FLASHFRAMES, frame);
		
		frameFlashManager:SetScript("OnUpdate", UICoreFrameFlash_OnUpdate);
	end
end

-- Called every frame to update flashing frames
function UICoreFrameFlash_OnUpdate(self, elapsed)
	local frame;
	local index = #FLASHFRAMES;
	
	-- Update timers for all synced frames
	for syncId, timer in pairs(UIFrameFlashTimers) do
		UIFrameFlashTimers[syncId] = timer + elapsed;
	end
	
	while FLASHFRAMES[index] do
		frame = FLASHFRAMES[index];
		frame._flashTimer = frame._flashTimer + elapsed;

		if ( (frame._flashTimer > frame._flashDuration) and frame._flashDuration ~= -1 ) then
			UICoreFrameFlashStop(frame);
		else
			local flashTime = frame._flashTimer;
			local alpha;
			
			if (frame._syncId) then
				flashTime = UIFrameFlashTimers[frame._syncId];
			end
			
			flashTime = flashTime%(frame._fadeInTime+frame._fadeOutTime+(frame._flashInHoldTime or 0)+(frame._flashOutHoldTime or 0));
			if (flashTime < frame._fadeInTime) then
				alpha = flashTime/frame._fadeInTime;
			elseif (flashTime < frame._fadeInTime+(frame._flashInHoldTime or 0)) then
				alpha = 1;
			elseif (flashTime < frame._fadeInTime+(frame._flashInHoldTime or 0)+frame._fadeOutTime) then
				alpha = 1 - ((flashTime - frame._fadeInTime - (frame._flashInHoldTime or 0))/frame._fadeOutTime);
			else
				alpha = 0;
			end
			
			frame:SetAlpha(alpha);
			frame:Show();
		end
		
		-- Loop in reverse so that removing frames is safe
		index = index - 1;
	end
	
	if ( #FLASHFRAMES == 0 ) then
		self:SetScript("OnUpdate", nil);
	end
end

-- Function to see if a frame is already flashing
function UICoreFrameIsFlashing(frame)
	for index, value in pairs(FLASHFRAMES) do
		if ( value == frame ) then
			return 1;
		end
	end
	return nil;
end

-- Function to stop flashing
function UICoreFrameFlashStop(frame)
	tDeleteItem(FLASHFRAMES, frame);
	frame:SetAlpha(1.0);
	frame._flashTimer = nil;
	if (frame._syncId) then
		UIFrameFlashTimerRefCount[frame._syncId] = UIFrameFlashTimerRefCount[frame._syncId]-1;
		if (UIFrameFlashTimerRefCount[frame._syncId] == 0) then
			UIFrameFlashTimers[frame._syncId] = nil;
			UIFrameFlashTimerRefCount[frame._syncId] = nil;
		end
		frame._syncId = nil;
	end
	if ( frame._showWhenDone ) then
		frame:Show();
	else
		frame:Hide();
	end
end
